<?php

namespace Larayellow\Newsletter\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Larayellow\Newsletter\Forms\Newsletter as NewsletterForm;
use Larayellow\Newsletter\Http\Requests\NewsletterRequest;
use Larayellow\Newsletter\Interfaces\NewsletterRepositoryInterface;
use Larayellow\Newsletter\Repositories\Eloquent\Filters\NewsletterResourceFilter;
use Larayellow\Newsletter\Repositories\Eloquent\Presenters\NewsletterListPresenter;

/**
 * Resource controller class for newsletter.
 */
class NewsletterResourceController extends BaseController
{

    /**
     * Initialize newsletter resource controller.
     *
     *
     * @return null
     */
    public function __construct(NewsletterRepositoryInterface $newsletter)
    {
        parent::__construct();
        $this->form = NewsletterForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('larayellow.newsletter.modules'), 'newsletter', guard_url('newsletter'));
        $this->repository = $newsletter;
    }

    /**
     * Display a list of newsletter.
     *
     * @return Response
     */
    public function index(NewsletterRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(NewsletterResourceFilter::class)
            ->setPresenter(NewsletterListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('newsletter::newsletter.names'))
            ->view('newsletter::newsletter.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display newsletter.
     *
     * @param Request $request
     * @param Model   $newsletter
     *
     * @return Response
     */
    public function show(NewsletterRequest $request, NewsletterRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('newsletter::newsletter.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('newsletter::newsletter.show')
            ->output();
    }

    /**
     * Show the form for creating a new newsletter.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(NewsletterRequest $request, NewsletterRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('newsletter::newsletter.name'))
            ->view('newsletter::newsletter.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new newsletter.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(NewsletterRequest $request, NewsletterRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('newsletter::newsletter.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('newsletter/newsletter/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/newsletter/newsletter'))
                ->redirect();
        }

    }

    /**
     * Show newsletter for editing.
     *
     * @param Request $request
     * @param Model   $newsletter
     *
     * @return Response
     */
    public function edit(NewsletterRequest $request, NewsletterRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('newsletter::newsletter.name'))
            ->view('newsletter::newsletter.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the newsletter.
     *
     * @param Request $request
     * @param Model   $newsletter
     *
     * @return Response
     */
    public function update(NewsletterRequest $request, NewsletterRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('newsletter::newsletter.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('newsletter/newsletter/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('newsletter/newsletter/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the newsletter.
     *
     * @param Model   $newsletter
     *
     * @return Response
     */
    public function destroy(NewsletterRequest $request, NewsletterRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('newsletter::newsletter.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('newsletter/newsletter/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('newsletter/newsletter/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
